/*
 * Copyright (c) 2014 Cloudera, Inc. All rights reserved.
 */

package com.cloudera.keytrustee.test;

//
// import com.cedarsoft.test.utils.CatchAllExceptionsRule;
import com.cloudera.keytrustee.TrusteeKeyProvider;
import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.crypto.key.KeyProvider;
import org.junit.Assert;
import org.junit.Rule;
import org.junit.Test;

import java.io.IOException;
import java.security.NoSuchAlgorithmException;

import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.TimeUnit;

/*
 * Tests that TrusteeKeyProvider behaves properly when used by multiple threads.
 */
public class ThreadedClientTest extends TestHelper {

  //This rule catches exceptions on all threads and fails the test if such exceptions are caught
  //@author Johannes Schneider (<a href="mailto:js@cedarsoft.com">js@cedarsoft.com</a>)
  //http://blog.cedarsoft.com/2011/12/junit-rule-fail-tests-on-exceptionsfailed-assertions-in-other-threads/
  //@Rule
  //public CatchAllExceptionsRule catchAllExceptionsRule = new CatchAllExceptionsRule();

  public ThreadedClientTest() {
    super();
  }

  @Test
  public void testThreadedProvider() {
    TrusteeKeyProvider tkp = super.registerNewProvider();
    ExecutorService executor = Executors.newFixedThreadPool(8);

    executor.execute(new ExerciseThread(tkp, 1));
    executor.execute(new ExerciseThread(tkp, 2));
    executor.execute(new ExerciseThread(tkp, 3));
    executor.execute(new ExerciseThread(tkp, 4));
    executor.execute(new ExerciseThread(tkp, 5));
    executor.execute(new ExerciseThread(tkp, 6));
    executor.execute(new ExerciseThread(tkp, 7));
    executor.execute(new ExerciseThread(tkp, 8));

    executor.shutdown();

    try {
      executor.awaitTermination(300, TimeUnit.SECONDS);
    } catch (InterruptedException e) {
      e.printStackTrace();
    }
  }

  protected String getUniqueSuffix() {
    long time = System.currentTimeMillis();
    long id = Thread.currentThread().getId();
    return String.valueOf(time) + String.valueOf(id);
  }

  public class ExerciseThread implements Runnable {

    private TrusteeKeyProvider tkp;

    public ExerciseThread(TrusteeKeyProvider tkp, int threadNumber) {
      this.tkp = tkp;
    }

    public void run() {
      for (int i = 0; i < 10; i++) {
        createKey();
        getCurrentKey();
      }
    }

    private void createKey() {
      try {
        byte[] material = getKeyMaterial(DEFAULT_ALGORITHM, DEFAULT_BITLENGTH);

        String name = "ThreadedClientTest.createKey-" + getUniqueSuffix();

        Configuration conf = getDefaultConfiguration();
        KeyProvider.Options options = new KeyProvider.Options(conf);

        KeyProvider.KeyVersion keyVersion = this.tkp.createKey(name, material, options);

        String outputName = keyVersion.getName();
        byte[] outputMaterial = keyVersion.getMaterial();
        String outputVersionName = keyVersion.getVersionName();

        Assert.assertEquals("Key name returned from provider doesn't match input name.", name, outputName);
        Assert.assertEquals("Key material length returned from provider doesn't match input material length.", material.length, outputMaterial.length);
        Assert.assertArrayEquals("Key material returned from provider doesn't match input material.", material, outputMaterial);

      } catch (NoSuchAlgorithmException e) {
        throw new RuntimeException(e);
      } catch (IOException e) {
        throw new RuntimeException(e);
      }
    }

    private void getCurrentKey() {
      try {
        String name = "testGetCurrentKey-" + getUniqueSuffix();
        byte[] material0 = getKeyMaterial(DEFAULT_ALGORITHM, DEFAULT_BITLENGTH);
        byte[] material1 = getKeyMaterial(DEFAULT_ALGORITHM, DEFAULT_BITLENGTH);
        byte[] material2 = getKeyMaterial(DEFAULT_ALGORITHM, DEFAULT_BITLENGTH);

        Configuration conf = getDefaultConfiguration();
        KeyProvider.Options options = new KeyProvider.Options(conf);

        KeyProvider.KeyVersion keyVersion0 = this.tkp.createKey(name, material0, options);
        KeyProvider.KeyVersion keyVersion1 = this.tkp.rollNewVersion(name, material1);
        KeyProvider.KeyVersion keyVersion2 = this.tkp.rollNewVersion(name, material2);

        KeyProvider.KeyVersion keyVersionActual = this.tkp.getCurrentKey(name);
        Assert.assertEquals("Key material from provider doesn't match material of last key version.", material2, keyVersion2.getMaterial());

      } catch (NoSuchAlgorithmException e) {
        throw new RuntimeException(e);
      } catch (IOException e) {
        throw new RuntimeException(e);
      }
    }
  }


}
